@@ -1,8 +1,15 @@
 #include "tests_main.h"
 
+extern "C" {
+const char * Version = "stub unit test Version string";
+};
+
 int main(int argc, char **argv) {
 	::testing::InitGoogleTest(&argc, argv);
 
+	init_lib();
+	init_auth();
+
 	// Some tests makes use of extra parameters passed to the tests
 	// executable. Save these params as static members of the base class.
 	if (argc > 1) {
