@@ -6,7 +6,7 @@
 
 #include "unity.h"
 
-void test_RegularPositive() {
+void test_RegularPositive(void) {
 	const char *str = "305";
 	u_long actual;
 
@@ -14,28 +14,28 @@ void test_RegularPositive() {
 	TEST_ASSERT_EQUAL(305, actual);
 }
 
-void test_PositiveOverflowBoundary() {
+void test_PositiveOverflowBoundary(void) {
 	const char *str = "4294967296";
 	u_long actual;
 
 	TEST_ASSERT_FALSE(atouint(str, &actual));
 }
 
-void test_PositiveOverflowBig() {
+void test_PositiveOverflowBig(void) {
 	const char *str = "8000000000";
 	u_long actual;
 
 	TEST_ASSERT_FALSE(atouint(str, &actual));
 }
 
-void test_Negative() {
+void test_Negative(void) {
 	const char *str = "-1";
 	u_long actual;
 
 	TEST_ASSERT_FALSE(atouint(str, &actual));
 }
 
-void test_IllegalChar() {
+void test_IllegalChar(void) {
 	const char *str = "50c3";
 	u_long actual;
 
